@@ -3,9 +3,9 @@
 using namespace Rcpp;
 
 arma::vec fptpdf(arma::vec rt, double A, double b, double mean_v, double sd_v,
-                 double t0, bool posdrift)
+                 double t0, double st0, bool posdrift)
 {
-  lba * obj = new lba(A, b, mean_v, sd_v, t0, posdrift, rt);
+  lba * obj = new lba(A, b, mean_v, sd_v, t0, st0, posdrift, rt);
   arma::vec out(obj->m_nrt);
 
   if(!obj->ValidateParams(false))
@@ -22,9 +22,9 @@ arma::vec fptpdf(arma::vec rt, double A, double b, double mean_v, double sd_v,
 }
 
 arma::vec fptcdf(arma::vec rt, double A, double b, double mean_v, double sd_v,
-                 double t0, bool posdrift)
+                 double t0, double st0, bool posdrift)
 {
-  lba * obj = new lba(A, b, mean_v, sd_v, t0, posdrift, rt);
+  lba * obj = new lba(A, b, mean_v, sd_v, t0, st0, posdrift, rt);
   arma::vec out(obj->m_nrt);
 
   if(!obj->ValidateParams(false))
@@ -41,7 +41,8 @@ arma::vec fptcdf(arma::vec rt, double A, double b, double mean_v, double sd_v,
 }
 
 arma::vec n1PDFfixedt0(arma::vec rt, arma::vec A, arma::vec b, arma::vec mean_v,
-                       arma::vec sd_v, arma::vec t0, bool posdrift) {
+                       arma::vec sd_v, arma::vec t0, arma::vec st0,
+                       bool posdrift) {
 
   unsigned int nmean_v = mean_v.n_elem;  // Number of accumulators/responses.
   unsigned int n       = rt.n_elem;      // Number of trials
@@ -49,14 +50,16 @@ arma::vec n1PDFfixedt0(arma::vec rt, arma::vec A, arma::vec b, arma::vec mean_v,
   unsigned int nA      = A.n_elem;
   unsigned int nb      = b.n_elem;
   unsigned int nt0     = t0.n_elem;
+  unsigned int nst0    = st0.n_elem;      // reduntant
 
   if (nsd_v == 1) sd_v = arma::repmat(sd_v, nmean_v, 1);
   if (nA    == 1) A    = arma::repmat(A,    nmean_v, 1);
   if (nb    == 1) b    = arma::repmat(b,    nmean_v, 1);
   if (nt0   == 1) t0   = arma::repmat(t0,   nmean_v, 1);
+  if (nst0  == 1) st0  = arma::repmat(st0,  nmean_v, 1);
 
   arma::vec onevec = arma::ones<arma::vec>(n);
-  arma::vec node1den = fptpdf(rt, A[0], b[0], mean_v[0], sd_v[0], t0[0],
+  arma::vec node1den = fptpdf(rt, A[0], b[0], mean_v[0], sd_v[0], t0[0], st0[0],
                               posdrift);
 
   if (nmean_v > 1)
@@ -64,7 +67,7 @@ arma::vec n1PDFfixedt0(arma::vec rt, arma::vec A, arma::vec b, arma::vec mean_v,
     for (size_t i = 1; i < nmean_v; i++)
     {
       node1den = node1den % (onevec - fptcdf(rt, A[i], b[i], mean_v[i],
-                                             sd_v[i], t0[i], posdrift));
+                                             sd_v[i], t0[i], st0[i], posdrift));
     }
   }
 
